/** @odoo-module **/

import { patch } from '@web/core/utils/patch'

import { TimeOffCalendarFilterPanel } from '@hr_holidays/views/calendar/filter_panel/calendar_filter_panel'
import { serializeDate } from '@web/core/l10n/dates'
import { onWillStart, onWillUpdateProps } from '@odoo/owl'

patch(TimeOffCalendarFilterPanel.prototype, {
  /**
   * Add onWillStart to load data from the holidays.calendar model to the dates displayed in the calendar, add onWillUpdateProps to update data when switching month, week or year
   */
  setup() {
    super.setup(...arguments)
    onWillStart(this.countHolidays)
    onWillUpdateProps(this.countHolidays)
  },

  /**
   * The function counts the number of working days and days off
   */
  async countHolidays() {
    const rangeStart = this.props.model.rangeStart
    const rangeEnd = this.props.model.rangeEnd
    const firstDay = await this.getFirstMonthDay(rangeStart, rangeEnd)
    if (!this.weekScale) {
      const lastDay = await this.getLastMonthDay(firstDay, rangeStart, rangeEnd)
      this.daysTotalAmount =
        (new Date(lastDay) - new Date(firstDay)) / (1000 * 60 * 60 * 24) + 1
      const unusualDays = this.getUnusualDaysList(firstDay, lastDay)
      await this.getAllHolidays(firstDay, lastDay)
      const nonWorkingDaysAndHolidaysSet = new Set([
        ...unusualDays,
        ...this.nonWorkingDays,
        ...this.holidays
      ])
      const nonWorkingDaysAndHolidaysSetWithoutRescheduled = new Set(
        [...nonWorkingDaysAndHolidaysSet].filter(
          value => !new Set(this.rescheduledWorkDays).has(value)
        )
      )
      this.workingDaysAmount =
        this.daysTotalAmount -
        nonWorkingDaysAndHolidaysSetWithoutRescheduled.size
      this.nonWorkingDaysAndHolidays =
        nonWorkingDaysAndHolidaysSetWithoutRescheduled.size
    }
  },

  /**
   * The function counts the number of working days, holidays and non-working days
   */
  async getAllHolidays(firstDay, lastDay) {
    const [holidaysList, nonWorkingDaysList, rescheduledWorkDaysList] =
      await this.orm.call('holidays.calendar', 'get_all_holidays', [
        firstDay,
        lastDay
      ])
    this.holidays = holidaysList
    this.nonWorkingDays = nonWorkingDaysList
    this.rescheduledWorkDays = rescheduledWorkDaysList
  },

  /**
   * The function defines the first day of the month
   */
  getFirstMonthDay(rangeStart, rangeEnd) {
    if ((rangeEnd - rangeStart) / (1000 * 60 * 60 * 24) > 7) {
      this.weekScale = false
      let newRangeStartMonth = 1
      let newRangeStartYear = rangeStart.year
      if (rangeStart.month + 1 <= 12) {
        newRangeStartMonth =
          rangeStart.month + 1 > 9
            ? `${rangeStart.month + 1}`
            : `0${rangeStart.month + 1}`
      } else {
        newRangeStartMonth = '01'
        newRangeStartYear += 1
      }
      return rangeStart.day === 1
        ? serializeDate(rangeStart, 'datetime')
        : `${newRangeStartYear}-${newRangeStartMonth}-01`
    } else {
      this.weekScale = true
    }
  },

  /**
   * The function determines the last day of the month
   */
  getLastMonthDay(firstDay, rangeStart, rangeEnd) {
    let lastDay = new Date(firstDay)
    // 42 is the number of days displayed on the calendar on a monthly scale.
    // In this case, we check that the scale of the year is displayed
    if ((rangeEnd - rangeStart) / (1000 * 60 * 60 * 24) > 42) {
      return serializeDate(rangeEnd, 'datetime')
    } else {
      lastDay.setMonth(lastDay.getMonth() + 1)
      lastDay.setDate(0)
      const newLastDayMonth =
        lastDay.getMonth() + 1 > 9
          ? `${lastDay.getMonth() + 1}`
          : `0${lastDay.getMonth() + 1}`
      return `${lastDay.getFullYear()}-${newLastDayMonth}-${lastDay.getDate()}`
    }
  },

  /**
   * The function gets a list of days off in a given period
   */
  getUnusualDaysList(firstDay, lastDay) {
    const newFirstDay = new Date(firstDay)
    const newLastDay = new Date(lastDay)
    const unusualDaysList = this.props.model.unusualDays
    const unusualDaysFiltered = unusualDaysList.filter(day => {
      const newDate = new Date(day)
      return (newDate >= newFirstDay) & (newDate <= newLastDay)
    })
    return unusualDaysFiltered
  }
})
